@@ -13,38 +13,18 @@ case "$ODL_RELEASE_BASE" in
     latest-snapshot)
         ODL_RELEASE=latest-snapshot
         ;;
+    fluorine-snapshot)
+        ODL_RELEASE=fluorine-snapshot-0.9
+        ;;
     oxygen-snapshot)
         ODL_RELEASE=oxygen-snapshot-0.8
         ;;
-    nitrogen-snapshot)
-        ODL_RELEASE=nitrogen-snapshot-0.7
-        ;;
-    carbon-snapshot)
-        ODL_RELEASE=carbon-snapshot-0.6
-        ;;
-    boron-snapshot)
-        # NOTE(yamahata): boron release ends by SR4. no more snapshot
-        # https://wiki.opendaylight.org/view/Simultaneous_Release:Boron_Release_Plan
-        ODL_RELEASE=boron-0.5.4-SR4
-        ;;
     *)
         echo "Unknown ODL release base: $ODL_RELEASE_BASE"
         exit 1
         ;;
 esac
 
-if [[ -z "$ODL_GATE_V2DRIVER" ]] && [[ -n "${RALLY_SCENARIO}" ]]; then
-    ODL_GATE_V2DRIVER=v2driver
-fi
-case "$ODL_GATE_V2DRIVER" in
-    v2driver)
-        ODL_V2DRIVER=True
-        ;;
-    v1driver|*)
-        ODL_V2DRIVER=False
-        ;;
-esac
-
 ODL_PORT_BINDING_CONTROLLER=pseudo-agentdb-binding
 
 ODL_GATE_SERVICE_PROVIDER=${ODL_GATE_SERVICE_PROVIDER%-}
@@ -66,11 +46,6 @@ case "$ODL_GATE_SERVICE_PROVIDER" in
 esac
 
 ODL_NETVIRT_KARAF_FEATURE=$ODL_NETVIRT_KARAF_FEATURE,odl-neutron-logger
-case "$ODL_RELEASE_BASE" in
-    carbon-snapshot|nitrogen-snapshot)
-        ODL_NETVIRT_KARAF_FEATURE=$ODL_NETVIRT_KARAF_FEATURE,odl-neutron-hostconfig-ovs
-        ;;
-esac
 
 local localrc_file=$DEVSTACK_PATH/local.conf
 
@@ -79,9 +54,6 @@ localrc_set $localrc_file "IS_GATE" "True"
 # Set here the ODL release to use for the Gate job
 localrc_set $localrc_file "ODL_RELEASE" "${ODL_RELEASE}"
 
-# Set here which driver, v1 or v2 driver
-localrc_set $localrc_file "ODL_V2DRIVER" "${ODL_V2DRIVER}"
-
 # Set timeout in seconds for http client to ODL neutron northbound
 localrc_set $localrc_file "ODL_TIMEOUT" "60"
 
@@ -117,6 +89,6 @@ localrc_set $localrc_file "RALLY_SCENARIO" "${RALLY_SCENARIO}"
 # delete and recreate network to workaroud netvirt bug:
 # https://bugs.opendaylight.org/show_bug.cgi?id=7456
 # https://bugs.opendaylight.org/show_bug.cgi?id=8133
-if [[ "$DEVSTACK_GATE_TOPOLOGY" == "multinode" ]] || [[ "$ODL_GATE_SERVICE_PROVIDER" == "vpnservice" ]]; then
+if [[ "$DEVSTACK_GATE_TOPOLOGY" == "multinode" ]]; then
     purge_and_recreate_initial_networks $DEVSTACK_PATH
 fi
