@@ -20,6 +20,7 @@ import mock
 import requests
 import testscenarios
 
+from neutron.db import api as db_api
 from neutron.db.models import securitygroup
 from neutron.db import segments_db
 from neutron.plugins.ml2 import plugin
@@ -27,6 +28,7 @@ from neutron.tests.unit import testlib_api
 from neutron_lib.api.definitions import multiprovidernet as mpnet_apidef
 from neutron_lib.api.definitions import provider_net as providernet
 from neutron_lib import constants as n_constants
+from neutron_lib.plugins import constants as plugin_const
 from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_config import fixture as config_fixture
@@ -38,6 +40,7 @@ from networking_odl.common import constants as odl_const
 from networking_odl.common import filters
 from networking_odl.common import utils
 from networking_odl.db import db
+from networking_odl.journal import base_driver
 from networking_odl.journal import cleanup
 from networking_odl.journal import journal
 from networking_odl.ml2 import mech_driver_v2
@@ -176,6 +179,14 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
         self.mech = mech_driver_v2.OpenDaylightMechanismDriver()
         self.mech.initialize()
 
+    def test_registered_plugin_type(self):
+        self.assertEqual(self.mech.plugin_type, plugin_const.CORE)
+
+    def test_registered_resources(self):
+        for resource in self.mech.RESOURCES:
+            self.assertIn(resource, base_driver.ALL_RESOURCES)
+            self.assertEqual(base_driver.ALL_RESOURCES[resource], self.mech)
+
     def _get_mock_network_operation_context(self):
         current = {'status': 'ACTIVE',
                    'subnets': [],
@@ -237,7 +248,7 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
             return_value=SECURITY_GROUP)
         _plugin.get_port = mock.Mock(return_value=current)
         _plugin.get_network = mock.Mock(return_value=_network)
-        _plugin_context_mock = {'session': self.db_session}
+        _plugin_context_mock = {'session': self.db_context.session}
         _network_context_mock = {'_network': _network}
         context = {'current': AttributeDict(current),
                    '_plugin': _plugin,
@@ -322,12 +333,12 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                     plugin_context.session.add(sg)
                     sg_dict = dict(sg)
                     sg_dict['security_group_rules'] = []
-                    with self.db_session.begin(subtransactions=True):
+                    with db_api.context_manager.writer.using(plugin_context):
                         self.mech.sync_from_callback_precommit(
                             plugin_context, operation, res_type, res_id,
                             context_, security_group=sg_dict)
                 if operation == odl_const.ODL_DELETE:
-                    with self.db_session.begin(subtransactions=True):
+                    with db_api.context_manager.writer.using(plugin_context):
                         self.mech.sync_from_callback_precommit(
                             plugin_context, operation, res_type, res_id,
                             context_,
@@ -336,26 +347,26 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                             security_group_rule_ids=[SG_RULE_FAKE_ID])
             elif (object_type == odl_const.ODL_SG_RULE and
                   operation == odl_const.ODL_DELETE):
-                with self.db_session.begin(subtransactions=True):
+                with db_api.context_manager.writer.using(plugin_context):
                     self.mech.sync_from_callback_precommit(
                         plugin_context, operation, res_type, res_id,
                         context_, security_group_id=SG_FAKE_ID)
             else:
-                with self.db_session.begin(subtransactions=True):
+                with db_api.context_manager.writer.using(plugin_context):
                     self.mech.sync_from_callback_precommit(
                         plugin_context, operation, res_type, res_id,
                         context_)
         else:
             method = getattr(self.mech, '%s_%s_precommit' % (operation,
                                                              object_type))
-            with self.db_session.begin(subtransactions=True):
+            with db_api.context_manager.writer.using(context):
                 method(context)
 
     def _test_operation_object(self, operation, object_type):
         self._call_operation_object(operation, object_type)
 
         context = self._get_mock_operation_context(object_type)
-        row = db.get_oldest_pending_db_row_with_lock(self.db_session)
+        row = db.get_oldest_pending_db_row_with_lock(self.db_context)
         self.assertEqual(operation, row['operation'])
         self.assertEqual(object_type, row['object_type'])
         self.assertEqual(context.current['id'], row['object_uuid'])
@@ -402,20 +413,20 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
     def _test_object_type(self, object_type, delete_expected_calls=1):
         # Add and process create request.
         self._test_thread_processing(odl_const.ODL_CREATE, object_type)
-        rows = db.get_all_db_rows_by_state(self.db_session,
+        rows = db.get_all_db_rows_by_state(self.db_context,
                                            odl_const.COMPLETED)
         self.assertEqual(1, len(rows))
 
         # Add and process update request. Adds to database.
         self._test_thread_processing(odl_const.ODL_UPDATE, object_type)
-        rows = db.get_all_db_rows_by_state(self.db_session,
+        rows = db.get_all_db_rows_by_state(self.db_context,
                                            odl_const.COMPLETED)
         self.assertEqual(2, len(rows))
 
         # Add and process update request. Adds to database.
         self._test_thread_processing(odl_const.ODL_DELETE, object_type,
                                      delete_expected_calls)
-        rows = db.get_all_db_rows_by_state(self.db_session,
+        rows = db.get_all_db_rows_by_state(self.db_context,
                                            odl_const.COMPLETED)
         self.assertEqual(2 + delete_expected_calls, len(rows))
 
@@ -430,7 +441,7 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                                      expected_calls=2)
 
         # Verify both rows are now marked as completed.
-        rows = db.get_all_db_rows_by_state(self.db_session,
+        rows = db.get_all_db_rows_by_state(self.db_context,
                                            odl_const.COMPLETED)
         self.assertEqual(2, len(rows))
 
@@ -452,8 +463,8 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
 
         # Get pending row and mark as processing so that
         # this row will not be processed by journal thread.
-        row = db.get_all_db_rows_by_state(self.db_session, odl_const.PENDING)
-        db.update_db_row_state(self.db_session, row[0], odl_const.PROCESSING)
+        row = db.get_all_db_rows_by_state(self.db_context, odl_const.PENDING)
+        db.update_db_row_state(self.db_context, row[0], odl_const.PROCESSING)
 
         # Create the object_type database row and process.
         # Verify that object request is not processed because the
@@ -463,10 +474,10 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                                      expected_calls=0)
 
         # Verify that all rows are still in the database.
-        rows = db.get_all_db_rows_by_state(self.db_session,
+        rows = db.get_all_db_rows_by_state(self.db_context,
                                            odl_const.PROCESSING)
         self.assertEqual(1, len(rows))
-        rows = db.get_all_db_rows_by_state(self.db_session, odl_const.PENDING)
+        rows = db.get_all_db_rows_by_state(self.db_context, odl_const.PENDING)
         self.assertEqual(1, len(rows))
 
     def _test_parent_delete_pending_child_delete(self, parent, child):
@@ -480,17 +491,17 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
 
         # Get pending row and mark as processing and update
         # the last_retried time
-        row = db.get_all_db_rows_by_state(self.db_session,
+        row = db.get_all_db_rows_by_state(self.db_context,
                                           odl_const.PENDING)[0]
         row.last_retried = last_retried
-        db.update_db_row_state(self.db_session, row, odl_const.PROCESSING)
+        db.update_db_row_state(self.db_context, row, odl_const.PROCESSING)
 
         # Test if the cleanup marks this in the desired state
         # based on the last_retried timestamp
         cleanup.cleanup_processing_rows(self.db_context)
 
         # Verify that the Db row is in the desired state
-        rows = db.get_all_db_rows_by_state(self.db_session, expected_state)
+        rows = db.get_all_db_rows_by_state(self.db_context, expected_state)
         self.assertEqual(1, len(rows))
 
     def test_driver(self):
@@ -506,10 +517,10 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
 
         method = getattr(self.mech, 'create_port_precommit')
         method(context)
-        self.db_session.flush()
+        self.db_context.session.flush()
 
         # Verify that the Db row has a tenant
-        rows = db.get_all_db_rows_by_state(self.db_session, odl_const.PENDING)
+        rows = db.get_all_db_rows_by_state(self.db_context, odl_const.PENDING)
         self.assertEqual(1, len(rows))
         _network = self._get_mock_network_operation_context().current
         self.assertEqual(_network['tenant_id'], rows[0]['data']['tenant_id'])
@@ -626,7 +637,7 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
             sg.security_group_rules = [rule]
             kwargs = {'security_group': sg,
                       'security_group_rule_ids': [SG_RULE_FAKE_ID]}
-            with self.db_session.begin(subtransactions=True):
+            with db_api.context_manager.writer.using(self.db_context):
                 self.mech.sync_from_callback_precommit(
                     self.db_context, odl_const.ODL_DELETE,
                     callback._RESOURCE_MAPPING[odl_const.ODL_SG],
@@ -651,7 +662,7 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
             rule.security_group_id = SG_FAKE_ID
             kwargs = {'security_group_rule_id': SG_RULE_FAKE_ID,
                       'security_group_id': SG_FAKE_ID}
-            with self.db_session.begin(subtransactions=True):
+            with db_api.context_manager.writer.using(self.db_context):
                 self.mech.sync_from_callback_precommit(
                     self.db_context, odl_const.ODL_DELETE,
                     callback._RESOURCE_MAPPING[odl_const.ODL_SG_RULE],
@@ -665,7 +676,7 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
         alloc_pool = context.current['allocation_pools']
         self._call_operation_object(odl_const.ODL_UPDATE,
                                     odl_const.ODL_SUBNET)
-        row = db.get_oldest_pending_db_row_with_lock(self.db_session)
+        row = db.get_oldest_pending_db_row_with_lock(self.db_context)
         self.assertEqual(alloc_pool, row.data['allocation_pools'])
 
     def test_sync_multiple_updates(self):
@@ -675,12 +686,12 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
                                         odl_const.ODL_NETWORK)
 
         # get the last update row
-        rows = db.get_all_db_rows(self.db_session)
+        rows = db.get_all_db_rows(self.db_context)
         rows.sort(key=operator.attrgetter("seqnum"))
         first_row = rows[0]
 
         # change the state to processing
-        db.update_db_row_state(self.db_session, first_row,
+        db.update_db_row_state(self.db_context, first_row,
                                odl_const.PROCESSING)
 
         # create 1 more operation to trigger the sync thread
@@ -691,9 +702,9 @@ class OpenDaylightMechanismDriverTestCase(base_v2.OpenDaylightConfigBase):
 
         # validate that all the pending rows stays in 'pending' state
         # first row should be 'processing' because it was not processed
-        processing = db.get_all_db_rows_by_state(self.db_session, 'processing')
+        processing = db.get_all_db_rows_by_state(self.db_context, 'processing')
         self.assertEqual(1, len(processing))
-        rows = db.get_all_db_rows_by_state(self.db_session, 'pending')
+        rows = db.get_all_db_rows_by_state(self.db_context, 'pending')
         self.assertEqual(2, len(rows))
 
     def test_update_port_filter(self):
