@@ -417,6 +417,7 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
         self.useFixture(OpenDaylightAgentDBFixture())
         super(TestPseudoAgentDBBindingController, self).setUp()
         self.useFixture(fixture.CallbackRegistryFixture())
+        self.cfg = self.useFixture(config_fixture.Config())
 
         self.mgr = pseudo_agentdb_binding.PseudoAgentDBBindingController()
 
@@ -497,29 +498,21 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
             self.test_valid_segment[api.ID], pass_vif_type,
             pass_vif_details, status=n_const.PORT_STATUS_ACTIVE)
 
-    def _test_bind_port_succeed_when_agent_status(self, hconfig, agent_status):
-        hconfig['alive'] = agent_status
+    def _test_bind_port_failed_when_agent_dead(self, hconfig):
+        hconfig['alive'] = False
         port_context = self._fake_port_context(
             fake_segments=[self.test_invalid_segment, self.test_valid_segment],
             host_agents=[hconfig])
         self.mgr.bind_port(port_context)
-        port_context.set_binding.assert_called()
+        port_context.set_binding.assert_not_called()
 
-    def test_bind_port_succeed_when_agent_dead_vpp(self):
+    def test_bind_port_failed_when_agent_dead_vpp(self):
         hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_vpp)
-        self._test_bind_port_succeed_when_agent_status(hconfig, False)
+        self._test_bind_port_failed_when_agent_dead(hconfig)
 
-    def test_bind_port_succeed_when_agent_dead_ovs(self):
+    def test_bind_port_failed_when_agent_dead_ovs(self):
         hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_ovs)
-        self._test_bind_port_succeed_when_agent_status(hconfig, False)
-
-    def test_bind_port_succeed_when_agent_alive_vpp(self):
-        hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_vpp)
-        self._test_bind_port_succeed_when_agent_status(hconfig, True)
-
-    def test_bind_port_succeed_when_agent_alive_ovs(self):
-        hconfig = deepcopy(self.sample_hconf_str_tmpl_subs_ovs)
-        self._test_bind_port_succeed_when_agent_status(hconfig, True)
+        self._test_bind_port_failed_when_agent_dead(hconfig)
 
     def test_bind_port_with_vif_type_vhost_user_vpp(self):
         """test bind_port with vpp."""
@@ -640,7 +633,6 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
     def test_prepare_inital_port_status_no_websocket(
             self, mocked_add_provisioning_component):
-        odl_features.feature_set = set()
         port_ctx = self._fake_port_context(
             fake_segments=[self.test_valid_segment])
         initial_port_status = self.mgr._prepare_initial_port_status(port_ctx)
@@ -650,7 +642,12 @@ class TestPseudoAgentDBBindingController(base.DietTestCase):
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
     def test_prepare_inital_port_status_with_websocket(
             self, mocked_add_provisioning_component):
-        odl_features.feature_set.add(odl_features.OPERATIONAL_PORT_STATUS)
+        feature_json = """{"features": {"feature":
+                            [{"service-provider-feature":
+                            "neutron-extensions:operational-port-status"}]}}"""
+        self.cfg.config(odl_features_json=feature_json, group='ml2_odl')
+        self.addCleanup(odl_features.deinit)
+        odl_features.init()
         port_ctx = self._fake_port_context(
             fake_segments=[self.test_valid_segment])
         initial_port_status = self.mgr._prepare_initial_port_status(port_ctx)
