@@ -186,13 +186,16 @@ function _install_opendaylight {
     touch /$Q_PLUGIN_CONF_FILE
 
     NETWORKING_ODL_DIR=${NETWORKING_ODL_DIR:-$REPO_BASE/networking-odl}
-    ODL_V2DRIVER=${ODL_V2DRIVER:-True}
     Q_USE_PUBLIC_VETH=False
     ODL_DONT_WAIT_OVS_BR=True
     # openstack service provider isn't needed, only ODL neutron northbound
     # is necessary for functional test
     ODL_NETVIRT_KARAF_FEATURE=odl-neutron-service,odl-restconf-all,odl-aaa-authn,odl-dlux-core,odl-mdsal-apidocs,odl-neutron-logger
     ODL_BOOT_WAIT_URL=controller/nb/v2/neutron/networks
+    if [[ "$VENV" =~ "dsvm-fullstack" ]]; then
+        ODL_BOOT_WAIT_URL=restconf/operational/network-topology:network-topology/topology/netvirt:1
+        export ODL_NETVIRT_KARAF_FEATURE=odl-neutron-service,odl-restconf-all,odl-aaa-authn,odl-dlux-core,odl-mdsal-apidocs,odl-netvirt-openstack,odl-neutron-logger
+    fi
     source $NETWORKING_ODL_DIR/devstack/settings.odl
 
     local ODL_NEUTRON_NETWORK_WAIT_URL=controller/nb/v2/neutron/networks
@@ -208,6 +211,12 @@ function _install_opendaylight {
     fi
 
     enable_service odl-server
+    if [[ "$VENV" =~ "fullstack" ]]; then
+        enable_service odl-compute
+        # They are needed by configure_opendaylight_l3
+        FLOATING_RANGE=172.24.5.0/24
+        PUBLIC_NETWORK_GATEWAY=172.24.5.1
+    fi
     source $NETWORKING_ODL_DIR/devstack/plugin.sh stack install
     source $NETWORKING_ODL_DIR/devstack/plugin.sh stack post-config
 
@@ -217,7 +226,15 @@ function _install_opendaylight {
 
 function _install_post_devstack {
     echo_summary "Performing post-devstack installation"
-    _install_databases
+
+    if [[ "$VENV" =~ "dsvm-fullstack" ]]; then
+        # NOTE(yamahata):
+        # fullstack can use sqlite. not db server is needed in theory.
+        # _install_databases nopg
+        echo "don't install databases"
+    else
+        _install_databases
+    fi
 
     # networkign-odl devstack plugin requires infra
     _install_infra
@@ -226,14 +243,25 @@ function _install_post_devstack {
     if is_ubuntu; then
         install_package isc-dhcp-client
         install_package netcat-openbsd
+        install_package iputils-arping  # needed fullstack arping test
     elif is_fedora; then
         install_package dhclient
+        install_package arping  # needed fullstack arping test
     else
         exit_distro_not_supported "installing dhclient package"
     fi
 }
 
 
+function _configure_iptables_rules {
+    # For linuxbridge agent fullstack tests we need to add special rules to
+    # iptables for connection of agents to rabbitmq:
+    CHAIN_NAME="openstack-INPUT"
+    sudo iptables -n --list $CHAIN_NAME 1> /dev/null 2>&1 || CHAIN_NAME="INPUT"
+    sudo iptables -I $CHAIN_NAME -s 240.0.0.0/8 -p tcp -m tcp -d 240.0.0.0/8 --dport 5672 -j ACCEPT
+}
+
+
 function configure_host_for_func_testing {
     echo_summary "Configuring host for functional testing"
 
@@ -247,7 +275,7 @@ function configure_host_for_func_testing {
 }
 
 # This function has been added because it's called by the devstack scripts
-# but since functional is not stacking devstack entirely this
+# but since functional/fullstack are not stacking devstack entirely this
 # this function is never imported. Thus, the creation of this no-op function
 function conductor_conf {
     :
@@ -263,3 +291,7 @@ if [[ "$IS_GATE" != "True" ]]; then
         configure_host_for_func_testing
     fi
 fi
+
+if [[ "$VENV" =~ "dsvm-fullstack" ]]; then
+    _configure_iptables_rules
+fi
